package com.ylcm.sys.controller;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;

import com.alibaba.fastjson.JSONObject;
import com.ylcm.sys.vo.*;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ylcm.sys.common.Constants;
import com.ylcm.sys.form.AdTheatreAnalysisDetailQueryForm;
import com.ylcm.sys.form.AdTheatreAnalysisQueryForm;
import com.ylcm.sys.service.AdTheatreTvStatService;

import lombok.extern.slf4j.Slf4j;

import java.util.List;

@Slf4j
@Controller
@RequestMapping("/theatre_analysis")
public class TheatreAnalysisController extends BaseController{

	@Resource
	private AdTheatreTvStatService adTheatreTvStatService;
	
	@RequestMapping(value = "/index.do")
	public String brandAnalysis(ModelMap model) {
		/**
		 * 1.跳转到列表页面
		 */
		return "analysis/theatre_analysis";
	}
	
	@RequestMapping(value = "/ajax_list.do")
	public void ajaxSearchList(HttpServletResponse response, AdTheatreAnalysisQueryForm queryForm) {
		try {
			
			IPage<AdTheatreAnalysisVO> page = adTheatreTvStatService.getPage(queryForm);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", page);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	/**
	 *   央卫视电视剧收视明细
	 * @param response
	 * @param queryForm
	 */
	@RequestMapping(value = "/ajax_detail_list.do")
	public void ajaxDetailList(HttpServletResponse response, AdTheatreAnalysisDetailQueryForm queryForm) {
		try {
			
			IPage<AdTheatreAnalysisDetailVO> page = adTheatreTvStatService.getDetailPage(queryForm);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", page);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	/**
	 * 山东卫视电视剧收视分析
	 */
	@RequestMapping(value = "/ajax_tv_ratio.do")
	public void tvRatio(HttpServletResponse response, AdTheatreAnalysisDetailQueryForm queryForm) {
		try {

			AdTvRatioVO adTvRatioVO = adTheatreTvStatService.getTvRatioVO(queryForm);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", adTvRatioVO);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	/**
	 * 电视剧收视区域分步
	 */
	@RequestMapping(value = "/ajax_tv_zone.do")
	public void tvZone(HttpServletResponse response, AdTheatreAnalysisDetailQueryForm queryForm) {
		try {

			AdTvZoneVO adTvZoneVO = adTheatreTvStatService.getTvZoneVO(queryForm);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", adTvZoneVO);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	/**
	 * 电视剧产品投放分析
	 */
	@RequestMapping(value = "/ajax_tv_product.do")
	public void tvProduct(HttpServletResponse response, AdTheatreAnalysisDetailQueryForm queryForm) {
		try {

			List<AdTheatreProductDistVO> productVO = adTheatreTvStatService.getProductVO(queryForm);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", productVO);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	/**
	 * 电视剧体裁结构分析
	 */
	@RequestMapping(value = "/ajax_tv_theme.do")
	public void tvTheme(HttpServletResponse response, AdTheatreAnalysisDetailQueryForm queryForm) {
		try {

			List<AdThemeDistVO> themeVO = adTheatreTvStatService.getThemeDistVO(queryForm);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", themeVO);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	/**
	 * 剧场电视剧收视率排名
	 * @param response
	 * @param startDate
	 * @param endDate
	 */
	@RequestMapping(value = "/ajax_tvTop_list.do")
	public void ajaxTvTopList(HttpServletResponse response, String startDate, String endDate) {
		try {

			List<AdTheatreTopVO> list = adTheatreTvStatService.getTvTop(startDate, endDate);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", list);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	/**
	 * 卫视剧场排名
	 * @param response
	 * @param startDate
	 * @param endDate
	 */
	@RequestMapping(value = "/ajax_mediaTop_list.do")
	public void ajaxMediaTopList(HttpServletResponse response, String startDate, String endDate) {
		try {

			List<AdTheatreTopVO> list = adTheatreTvStatService.getMediaTop(startDate, endDate);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", list);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}

	@RequestMapping(value = "/ajax_topList.do")
	public void ajaxTopList(HttpServletResponse response, String startDate, String endDate) {
		try {
			List<AdTheatreTopVO> tvList = adTheatreTvStatService.getTvTop(startDate, endDate);
			List<AdTheatreTopVO> mediaList = adTheatreTvStatService.getMediaTop(startDate, endDate);
			JSONObject listJson = new JSONObject();
			listJson.put("tv", tvList);
			listJson.put("media", mediaList);
			/**
			 * 3.返回结果
			 */
			JSONObject jsonobj = new JSONObject();
			jsonobj.put("data", listJson);
			jsonobj.put("code", Constants.CODE_SUCCESS);
			ajaxJson(jsonobj.toString(), response);
		} catch (Exception e) {
			log.error(e.getMessage(), e);
			ajaxJson("{\"code\":\"0\",\"msg\":\"获取列表失败\"}", response);
		}
	}
}
