package com.ylcm.sys.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.ylcm.sys.domain.AdTheatre;
import com.ylcm.sys.domain.AdTheatreTv;
import com.ylcm.sys.form.AdTheatreForm;
import com.ylcm.sys.mapper.AdTheatreMapper;
import com.ylcm.sys.service.AdTheatreService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ylcm.sys.service.AdTheatreTvService;
import com.ylcm.sys.vo.AdTheatreVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;

/**
 * <p>
 * 剧场管理表 服务实现类
 * </p>
 *
 * @author admin
 * @since 2020-07-01
 */
@Service
public class AdTheatreServiceImpl extends ServiceImpl<AdTheatreMapper, AdTheatre> implements AdTheatreService {

    @Resource
    private AdTheatreMapper adTheatreMapper;
    @Resource
    private AdTheatreTvService adTheatreTvService;

    @Override
    public IPage<AdTheatreVO> page(IPage<AdTheatre> page, AdTheatreForm form) {
        return adTheatreMapper.getListByCondition(page, form);
    }

    @Override
    public AdTheatre getUniqu(AdTheatre params) {
        LambdaQueryWrapper<AdTheatre> queryWrapper = new LambdaQueryWrapper<AdTheatre>();
        queryWrapper.eq(AdTheatre::getName, params.getName());
        queryWrapper.eq(AdTheatre::getMediaName, params.getMediaName());
        queryWrapper.eq(AdTheatre::getMediaNum, params.getMediaNum());
        queryWrapper.eq(AdTheatre::getShowDate, params.getShowDate());
        queryWrapper.eq(AdTheatre::getShowTime, params.getShowTime());
        return baseMapper.selectOne(queryWrapper);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public String batchAdd(List<AdTheatre> list, List<AdTheatreTv> tvList) throws Exception {
        if (list.size() != tvList.size()){
            throw new Exception("批量新增error：剧场ListSize != 收视率ListSize");
        }
        for (int i = 0; i < list.size(); i++){
            /*
             * insert 剧场数据
             */
            AdTheatre record = list.get(i);
            AdTheatre old = getUniqu(record);
            if (old != null) {
                throw new Exception("数据重复");
            }
            baseMapper.insert(record);
            AdTheatre newTheatre = getUniqu(record);
            if (newTheatre == null){
                throw new Exception("insert error：" + record.toString());
            }

            /*
             * insert 电视剧收视率数据
             */
            AdTheatreTv theatreTv = tvList.get(i);
            theatreTv.setAdTheatreId(newTheatre.getId());
            adTheatreTvService.add(theatreTv);
        }
        return "ok";
    }
}
