package com.ylcm.sys.service.impl;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;

import javax.annotation.Resource;

import com.ylcm.sys.domain.*;
import com.ylcm.sys.dto.AdTvQueryDTO;
import com.ylcm.sys.mapper.AdSpecialMapper;
import com.ylcm.sys.mapper.AdTheatreTvMapper;
import com.ylcm.sys.service.AdMediaService;
import com.ylcm.sys.vo.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.ylcm.sys.common.EnumHelperUtil;
import com.ylcm.sys.common.ProjectUtil;
import com.ylcm.sys.enums.AdTheatreTimeEnum;
import com.ylcm.sys.enums.MediaRankEnum;
import com.ylcm.sys.form.AdTheatreAnalysisDetailQueryForm;
import com.ylcm.sys.form.AdTheatreAnalysisQueryForm;
import com.ylcm.sys.mapper.AdTheatreMapper;
import com.ylcm.sys.mapper.AdTheatreTvStatMapper;
import com.ylcm.sys.service.AdTheatreTvStatService;

/**
 * <p>
 * 关联电视剧数据 服务实现类
 * </p>
 *
 * @author admin
 * @since 2020-07-01
 */
@Service
public class AdTheatreTvStatServiceImpl extends ServiceImpl<AdTheatreTvStatMapper, AdTheatreTvStat> implements AdTheatreTvStatService {

	@Resource
	private AdTheatreMapper adTheatreMapper;

	@Resource
	private AdMediaService adMediaService;

	@Resource
	private AdTheatreTvStatService adTheatreTvStatService;

	@Resource
	private AdTheatreTvMapper adTheatreTvMapper;

	@Resource
	private AdSpecialMapper adSpecialMapper;

	/**
	 * 计算 某个频道的收视率
	 * @param mediaName
	 * @param theatreTime
	 * @param startDate
	 * @param endDate
	 * @return
	 */
	@Cacheable(value = "TheatreAnalysis",keyGenerator = "cacheKeyGenerator")
	@Override
	public AdTheatreTvStat statByMedia(String mediaName, Integer theatreTime, String startDate, String endDate) {
		return baseMapper.statByMedia(mediaName, theatreTime, startDate, endDate);
	}

	@Cacheable(value = "TheatreAnalysis",keyGenerator = "cacheKeyGenerator")
	@Override
	public IPage<AdTheatreAnalysisVO> getPage(AdTheatreAnalysisQueryForm queryForm) {
		IPage<AdTheatreTvStat> adTheatreTvs = baseMapper.getPage(new Page<AdTheatreTvStat>(queryForm.getPageNo(), queryForm.getPageSize()), queryForm);
		IPage<AdTheatreAnalysisVO> analysisPage = new Page<AdTheatreAnalysisVO>();
		BeanUtils.copyProperties(adTheatreTvs, analysisPage);
		List<AdTheatreAnalysisVO> analysisVOs = new ArrayList<AdTheatreAnalysisVO>();
		if (CollectionUtils.isNotEmpty(adTheatreTvs.getRecords())) {
			for (AdTheatreTvStat adTheatreTvStat : adTheatreTvs.getRecords()) {
				AdTheatreAnalysisVO adTheatreAnalysisVO = new AdTheatreAnalysisVO();
				BeanUtils.copyProperties(adTheatreTvStat, adTheatreAnalysisVO);
				
				AdTheatreTimeEnum adTheatreTimeEnum = EnumHelperUtil.getOneByCode(AdTheatreTimeEnum.class, adTheatreTvStat.getTheatreTime() + "");
				adTheatreAnalysisVO.setTheatreTimeName(adTheatreTimeEnum.getName());
				
				MediaRankEnum mediaRankEnum = EnumHelperUtil.getOneByCode(MediaRankEnum.class, adTheatreTvStat.getChannelSystem() + "");
				adTheatreAnalysisVO.setChannelSystem(mediaRankEnum.getName());
				
				// 获取频道收视率
				AdTheatreTvStat mediaStat = adTheatreTvStatService.statByMedia(adTheatreTvStat.getMediaName(), adTheatreTvStat.getTheatreTime(), queryForm.getStartDate(), queryForm.getEndDate());
				adTheatreAnalysisVO.setMediaDk29(mediaStat.getDk29()/mediaStat.getTvNum());
				adTheatreAnalysisVO.setMediaRating29(round2(mediaStat.getRating29()/mediaStat.getTvNum()));
				
				adTheatreAnalysisVO.setEffect(ProjectUtil.getMediaEffect(adTheatreAnalysisVO.getRating29(), adTheatreAnalysisVO.getMediaRating29()));
				
				analysisVOs.add(adTheatreAnalysisVO);
			}
			analysisPage.setRecords(analysisVOs);
		}
		
		return analysisPage;
	}
	
	private float round2(float a) {
		float x = (float)(Math.round(a*100)/100.0);
		return x;
	}

	@Override
	public IPage<AdTheatreAnalysisDetailVO> getDetailPage(AdTheatreAnalysisDetailQueryForm queryForm) {
		AdTvQueryDTO adTvQueryDTO = AdTvQueryDTO.from(queryForm);
		LambdaQueryWrapper<AdTheatre> queryWrapper = new LambdaQueryWrapper<AdTheatre>();
		queryWrapper.eq(AdTheatre::getName, adTvQueryDTO.getName());
		queryWrapper.eq(AdTheatre::getMediaName, adTvQueryDTO.getMediaName());
		queryWrapper.ge(AdTheatre::getShowTime, adTvQueryDTO.getStartTime());
		queryWrapper.lt(AdTheatre::getShowTime, adTvQueryDTO.getEndTime());
		

		queryWrapper.ge(AdTheatre::getShowDate, adTvQueryDTO.getStartDate());
		queryWrapper.le(AdTheatre::getShowDate, adTvQueryDTO.getEndDate());

		IPage<AdTheatre> adTheatreTvs = adTheatreMapper.selectPage(new Page<AdTheatre>(queryForm.getPageNo(), queryForm.getPageSize()), queryWrapper);
		IPage<AdTheatreAnalysisDetailVO> analysisPage = new Page<AdTheatreAnalysisDetailVO>();
		BeanUtils.copyProperties(adTheatreTvs, analysisPage);
		
		
		List<AdTheatreAnalysisDetailVO> analysisVOs = new ArrayList<AdTheatreAnalysisDetailVO>();
		if (CollectionUtils.isNotEmpty(adTheatreTvs.getRecords())) {
			for (AdTheatre adTheatre : adTheatreTvs.getRecords()) {
				AdTheatreAnalysisDetailVO detailVO = new AdTheatreAnalysisDetailVO();
				BeanUtils.copyProperties(adTheatre, detailVO);

				detailVO.setProductName(adTheatre.getTypeName());
				detailVO.setJinxian(new BigDecimal(adTheatre.getJinxian()));

				// 省网数据
				LambdaQueryWrapper<AdTheatreTv> queryWrapperTv = new LambdaQueryWrapper<AdTheatreTv>().eq(AdTheatreTv::getAdTheatreId, adTheatre.getId());
				AdTheatreTv adTheatreTv = adTheatreTvMapper.selectOne(queryWrapperTv);
				if (adTheatreTv != null) {
					BeanUtils.copyProperties(adTheatreTv, detailVO);
				}
				// 广告
				LambdaQueryWrapper<AdSpecial> queryWrapperSp = new LambdaQueryWrapper<AdSpecial>()
						.eq(AdSpecial::getTvName, adTheatre.getName())
						.eq(AdSpecial::getMediaName, adTheatre.getMediaName())
						.eq(AdSpecial::getShowDate, adTheatre.getShowDate())
						.eq(AdSpecial::getShowTime, adTheatre.getShowTime());
				AdSpecial adSpecial = adSpecialMapper.selectOne(queryWrapperSp);
				if (adSpecial != null) {
					detailVO.setShowDuration(adSpecial.getShowDuration() + "");
				}

				analysisVOs.add(detailVO);
			}
			analysisPage.setRecords(analysisVOs);
		}
		
		return analysisPage;
	}
	
	public void saveStatByTheatreTv(AdTheatre adTheatre, AdTheatreTv adTheatreTv) {
		AdTheatreTvStat adTheatreTvStat = new AdTheatreTvStat();
		BeanUtils.copyProperties(adTheatre, adTheatreTvStat);

		adTheatreTvStat.setTheatreTime(ProjectUtil.getTypeByLocalTime(adTheatre.getShowTime()));
		adTheatreTvStat.setSeason(ProjectUtil.getSeasonByLocalDate(adTheatre.getShowDate()));
		adTheatreTvStat.setRating29(adTheatreTv.getRating29());
		adTheatreTvStat.setDk29(adTheatreTv.getDk29());

		LambdaQueryWrapper<AdTheatreTvStat> lambdaQueryWrapper = new LambdaQueryWrapper<AdTheatreTvStat>()
				.eq(AdTheatreTvStat::getName, adTheatreTvStat.getName())
				.eq(AdTheatreTvStat::getMediaName, adTheatreTvStat.getMediaName())
				.eq(AdTheatreTvStat::getTheatreTime, adTheatreTvStat.getTheatreTime())
				.eq(AdTheatreTvStat::getSeason, adTheatreTvStat.getSeason());
		AdTheatreTvStat domainInDb = baseMapper.selectOne(lambdaQueryWrapper);
		if (domainInDb != null) {
			baseMapper.insert(adTheatreTvStat);
		} else {
			BeanUtils.copyProperties(adTheatreTvStat, domainInDb);
			baseMapper.updateById(domainInDb);
		}
	}

	@Cacheable(value = "TheatreAnalysis",keyGenerator = "cacheKeyGenerator")
	@Override
	public List<AdTheatreTopVO> getTvTop(String startDate, String endDate) {
		List<AdTheatreTopVO> list = baseMapper.tvTop(startDate, endDate);
		list.stream().forEach(i -> i.init());
		return list;
	}

	@Cacheable(value = "TheatreAnalysis",keyGenerator = "cacheKeyGenerator")
	@Override
	public List<AdTheatreTopVO> getMediaTop(String startDate, String endDate) {
		List<AdTheatreTopVO> list = baseMapper.mediaTop(startDate, endDate);
		list.stream().forEach(i -> i.init());
		list.stream().forEach(i -> {
			MediaRankEnum mediaRankEnum = EnumHelperUtil.getOneByCode(MediaRankEnum.class, i.getMediaRank());
			i.setMediaRank(mediaRankEnum.getName());
		});
		return list;
	}

	@Override
	public AdTvRatioVO getTvRatioVO(AdTheatreAnalysisDetailQueryForm queryForm) {
		List<AdTvRatioTrendVO> trendVOS = baseMapper.tvRatioTrend(queryForm);
		AdTheatreTv adTheatreTv = baseMapper.tvRatioPeople(AdTvQueryDTO.from(queryForm));

		AdTvRatioVO adTvRatioVO = new AdTvRatioVO();
		adTvRatioVO.setTrendVOS(trendVOS);

		List<AdTvRatioPeopleVO> peopleVOS = AdTvRatioPeopleVO.from(adTheatreTv);
		adTvRatioVO.setPeopleVOS(peopleVOS);

		return adTvRatioVO;
	}

	@Override
	public AdTvZoneVO getTvZoneVO(AdTheatreAnalysisDetailQueryForm queryForm) {
		AdTheatreTv adTheatreTv = baseMapper.tvRatioZone(AdTvQueryDTO.from(queryForm));
		List<AdTvZonePeopleVO> tvZone = AdTvZonePeopleVO.from(adTheatreTv);

		// 去掉电视剧
		queryForm.setName(null);
		AdTheatreTv adTheatreMedia = baseMapper.tvRatioZone(AdTvQueryDTO.from(queryForm));
		List<AdTvZonePeopleVO> mediaZone = AdTvZonePeopleVO.from(adTheatreMedia);

		AdTvZoneVO adTvZoneVO = new AdTvZoneVO();
		adTvZoneVO.setTvNum(tvZone);
		adTvZoneVO.setMediaNum(mediaZone);

		return adTvZoneVO;
	}

	@Override
	public List<AdTheatreProductDistVO> getProductVO(AdTheatreAnalysisDetailQueryForm queryForm) {
		return baseMapper.productDist(AdTvQueryDTO.from(queryForm));
	}

	@Override
	public List<AdThemeDistVO> getThemeDistVO(AdTheatreAnalysisDetailQueryForm queryForm) {
		return baseMapper.themeDist(queryForm);
	}
}
